#!/usr/bin/perl
#
#Copyright Xinjiang University (Mijit Ablimit); Tsinghua University (Dong Wang, Xuewei Zhang)
#LICENCE: Apache 2.0
#
#Code transfer between Uyghur and Latin 
#usage:
#From latin to ugyhur
#cat <latin-file> | perl ucoding.pl  -cmap  auyghur.code
#e.g., cat ../../data_thuyg20/trans/trans.train | perl ucoding.pl   -cmap  auyghur.code | tee uyg.txt
#From ugyhur to latin
#cat <uyghur-file> | perl -reverse  -cmap auyghur.code 
#e.g., cat uyg.txt | perl ucoding.pl -reverse   -cmap  auyghur.code

binmode(STDIN,  ":utf8");
binmode(STDOUT, ":utf8");
binmode(STDERR, ":utf8");

# ////////////////////////////////////////////////////////////////////////////
%acodedict;
%ucodedict;   # a global hash table containing different Uyghur codes;

# //////////////////////////////////////////////////////////////////////////
$codemap="auyghur.code"; 
$rev = 0; #if reverse the map file

# Read command line parameters;
$flag = $ARGV[0]; 

if($flag eq "-cmap")  # code change with your own map file;
{
 $codemap = $ARGV[1];  
}elsif($flag eq "-reverse"){
 $rev = 1; 
}elsif($flag eq "-help"){
 usage();
}

CodeMap($codemap) || die "Error in code mapping";
CodeFlip();


#load code mapping 
sub CodeMap{
 my ($codeFile) = @_;  
 open CODEFILE, "<:utf8", $codeFile || die "Open mapping file failed";

 while(my $codeline = <CODEFILE>)
 {
  chomp $codeline; $codeline =~ s/\s+//g; 
   if(not $codeline){
    next;
   }

  my ($acode, $codeID, $ucode)= split(/=/, $codeline, 3);

  #reverse mapping
  if ($rev == 1) { ($ucode, $acode) = ($acode, $ucode); }

  if($acode  and $codeID )
  {  
   if(int($acode) eq $acode) { $acode = chr($acode);}  #  may used for double characters;
				   
   if(not $acodedict{$acode}){  
    $acodedict{$acode} =  $codeID; 
   }else {
    print STDERR "There are duplicated codes for $acode = $codeID in the code-map file.\n"; 
    close CODEFILE; 
    return 0;
   } 
   # but the codes being replaced are various, and many mapped to one ...
   for(split(/,/, $ucode))
   { 
    if($_) {               
     my $wild = $_; # may be used for double code chars... 
      if(int($wild) eq $wild) { $wild = chr($wild);  }
     if(not $ucodedict{$wild} )
     {
      $ucodedict{$wild}= $acode; # a character should be unique, but unfortunately many codes are used in various systems.
     }else { 
      print STDERR "There are ambiguous codes for character: $wild\n"; 
      close CODEFILE; 
      return 0; 
     }

    }
   }  
  }
 } 
close CODEFILE;

my $asize = keys %acodedict; 
my $usize = keys %ucodedict;
if($asize < 33 ) { 
 print STDERR "Only $asize characters are not enough, something is wrong with the uyghur code file.\n"; 
 return 0;
}
 print STDERR "Total $asize  Uyghur target codes, and $usize source codes are loaded.\n"; 


} 


# One to one code change between two sets of characters. 
# only many -> one code converting.....
sub CodeFlip {  

 my $linecount = 0;

 while(my $sourceline = <STDIN>)
 {
  chomp $sourceline;  $sourceline = $sourceline . "!"; # add a final indicator, never get to this one.
   $linecount++;
  my $destline = "";  my $codeindex = 0;
  my $token = "";
  my $linelen = length($sourceline) - 1; 
  while($linelen  > $codeindex)  
  { 
   my $char = substr($sourceline, $codeindex++, 1) ;
   if( ($linelen-$codeindex) > 2) 
   { 
    my $double = substr($sourceline, $codeindex-1, 2);
    if($ucodedict{$double}) { $char = $double; $codeindex++;  }
   }
   if ($ucodedict{$char})  {  
    $token = $token . $ucodedict{$char}; 
   }
   else
   {   
    if($token ne "")  { 
     $destline = $destline . $token . $char;  $token = "";  
    }
    else {   
     $destline = $destline . $char; 
    }
   }

  } 
  print STDOUT  "$destline\n";

 }        

 print STDERR "Code transferred for $linecount lines.\n";

} 


sub usage {

 my $me = $0 ; # $me =~ s,^.*/,,;
 $me =~ s/^.*\///;


 print "Change Latin chars to Uyghur or reverse. Read from stdin and output to stdout.\n";
 print "Usage: $me  [-cmap mapfile] [-rev] [-help]  \n";
 print "-cmap mapfile:  user-defined code-mapping, default is auyghur.code\n";
 print "-rev: reverse mapping: read in Uyghur and output Latin chars\n";

 exit(0);
}



